﻿using Abt.Controls.SciChart;
using Abt.Controls.SciChart.Model.DataSeries;
using Abt.Controls.SciChart.Visuals;
using Abt.Controls.SciChart.Visuals.Annotations;
using Abt.Controls.SciChart.Visuals.RenderableSeries;
using Microsoft.Practices.Prism.Mvvm;
using Microsoft.Practices.Prism.PubSubEvents;
using System;
using System.Collections.Generic;
using System.Linq;

namespace ExampleProject
{
  public struct DataPoint
  {
    public double XValue;
    public double YValue;
    public int Index;
  }

  public class ChartViewModel : BindableBase
  {
    private List<LineAnnotationWithLabelsBase> LineAnnotations = new List<LineAnnotationWithLabelsBase>();

    private XyDataSeries<double, double> dataCH0;
    private XyDataSeries<double, double> dataCH1;

    private BoxAnnotationCoordinates coordCH0 = new BoxAnnotationCoordinates();
    private BoxAnnotationCoordinates coordCH1 = new BoxAnnotationCoordinates();
    private DoubleRange chartXRange = new DoubleRange();
    private double currentPosition;

    private int selectedChartLine;

    private IDictionary<IRenderableSeries, List<DataPoint>> selectedDataPoints;
    private ChartCursorModifier corrShiftCursorModifier; // Cursorlinie der Korrekturwerte
    private TestElement currentElement;

    public TestElement CurrentElement
    {
      get
      {
        return currentElement;
      }

      set
      {
        currentElement = value;
        OnPropertyChanged("CurrentElement");
      }
    }

    /// <summary>
    /// Event Aggregator for sending to CtrlBaseViewModels
    /// </summary>

    public ChartCursorModifier CorrShiftCursorModifier
    {
      get
      {
        return corrShiftCursorModifier;
      }
    }

    public double CurrentPosition
    {
      get { return currentPosition; }
      set
      {
        SetProperty(ref currentPosition, value);
        UpdateBoxAnnotation(currentElement);
      }
    }

    /// <summary>
    /// Current selected line of chart
    /// </summary>
    public int SelectedChartLine
    {
      get { return selectedChartLine; }
      set { SetProperty(ref selectedChartLine, value); }
    }

    public DoubleRange ChartXRange
    {
      get { return chartXRange; }
      set
      {
        if (SetProperty(ref chartXRange, value))
        {
          //OnPropertyChanged("ScichartRangeMin");
          //OnPropertyChanged("ScichartRangeMax");
          //OnPropertyChanged("ScichartViewportSize");
          //OnPropertyChanged("ScichartScrollbarValue");

          if (corrShiftCursorModifier != null)
            corrShiftCursorModifier.SetLineToCurrentPointWithSelectingElement();
        }
      }
    }

    public SciChartSurface SciChart { get; private set; }

    public double ScichartRangeMin
    {
      get
      {
        return (double)SciChart.XAxis.GetMaximumRange().Min;
      }
    }

    public double ScichartRangeMax
    {
      get
      {
        return (double)SciChart.XAxis.GetMaximumRange().Max;
      }
    }

    public double ScichartScrollbarDirection
    {
      get
      {
        return 180;
      }
    }

    public double ScichartViewportSize
    {
      get
      {
        return ChartXRange.Diff;
      }
    }

    public double ScichartScrollbarValue
    {
      get
      {
        return ChartXRange.Min + ScichartViewportSize / 2;
      }
      set
      {
        DoubleRange range = new DoubleRange();

        range.Min = value - ScichartViewportSize / 2;
        range.Max = value + ScichartViewportSize / 2;

        ChartXRange = range;

        OnPropertyChanged("ScichartRangeMin");
        OnPropertyChanged("ScichartRangeMax");
        OnPropertyChanged("ScichartViewportSize");
        OnPropertyChanged("ScichartScrollbarValue");
      }
    }

    /// <summary>
    /// Dataserie of shifting data
    /// </summary>
    public XyDataSeries<double, double> DataCH0
    {
      get { return dataCH0; }
      set { SetProperty(ref dataCH0, value); }
    }

    /// <summary>
    /// Dataserie of lifting data
    /// </summary>
    public XyDataSeries<double, double> DataCH1
    {
      get { return dataCH1; }
      set { SetProperty(ref dataCH1, value); }
    }

    public BoxAnnotationCoordinates Coord_CH0
    {
      get { return coordCH0; }
      set { SetProperty(ref coordCH0, value); }
    }

    public BoxAnnotationCoordinates Coord_CH1
    {
      get { return coordCH1; }
      set { SetProperty(ref coordCH1, value); }
    }

    /// <summary>
    /// Gets or sets the SelectedDataPoints. This is bound to SimpleDataPointSelectionModifier.SelectedPoints
    /// </summary>
    public IDictionary<IRenderableSeries, List<DataPoint>> SelectedDataPoints
    {
      get { return selectedDataPoints; }
      set { SetProperty(ref selectedDataPoints, value); }
    }

    public ChartViewModel()
    {
    }

    public ChartViewModel(SciChartSurface sciChart, ChartCursorModifier cursorModifier)
      : this()
    {
      SciChart = sciChart;
      corrShiftCursorModifier = cursorModifier;
    }

    private TestList list1 = new TestList();

    public TestList List1
    {
      get { return list1; }
      set { list1 = value; }
    }

    private TestList list2 = new TestList();

    public TestList List2
    {
      get { return list2; }
      set { list2 = value; }
    }

    public void UpdateChart()
    {
      for (int i = 0; i < 1000; i++)
      {
        list1.Add(new TestElement()
          {
            StartPosition = i,
            Value = i,
          });

        list2.Add(new TestElement()
        {
          StartPosition = i,
          Value = i * 2,
        });
      }

      XyDataSeries<double, double> xydataShifting = new XyDataSeries<double, double>();
      XyDataSeries<double, double> xydataLifting = new XyDataSeries<double, double>();

      foreach (TestElement elem in list1)
        xydataShifting.Append(elem.StartPosition, elem.Value);

      foreach (TestElement elem in list2)
        xydataLifting.Append(elem.StartPosition, elem.Value);

      DataCH0 = xydataShifting;
      DataCH1 = xydataLifting;

      TestElement element = list1.First();
      ChartXRange = new DoubleRange(0, 200);
      corrShiftCursorModifier.SetVisibleRangeofChart(element);

      corrShiftCursorModifier.SetToBeginning(list1[3]);

      UpdateBoxAnnotation(element);

      OnPropertyChanged("ScichartRangeMin");

      OnPropertyChanged("ScichartRangeMax");

      OnPropertyChanged("ScichartViewportSize");

      OnPropertyChanged("ScichartScrollbarValue");

      OnPropertyChanged("ScichartScrollbarDirection");
    }

    private void UpdateBoxAnnotation(TestElement currentElement)
    {
      SetCoordCH0(currentElement, list1);
      SetCoordCH1(currentElement, list2);
    }

    /// <summary>
    /// Setzen der Koordinaten der BoxAnnotation für die Hebewerte
    /// </summary>
    /// <param name="data">Aktuelles Element</param>
    /// <param name="corr">Elementliste</param>
    public void SetCoordCH1(TestElement data, TestList corr)
    {
      double maxExtent = 1;   //min max yAxis;1 is default
      if (DataCH1 != null)
      {
        if (DataCH1.Count > 0)
        {
          DoubleRange range = DataCH1.YRange.AsDoubleRange();
          maxExtent = Math.Max(Math.Abs(range.Min), Math.Abs(range.Max));
        }

        Coord_CH1.X1 = data.StartPosition;
        Coord_CH1.X2 = data.StartPosition + 10;

        Coord_CH1.Y1 = maxExtent;
        Coord_CH1.Y2 = -maxExtent;

        if (maxExtent > Double.Epsilon && !Double.IsInfinity(maxExtent) && SciChart.YAxes.Count > 0)
        {
          SciChart.YAxes[1].VisibleRange = RangeFactory.NewWithMinMax(SciChart.YAxes[1].VisibleRange, -maxExtent, maxExtent);
          SciChart.YAxes[1].VisibleRangeLimit = RangeFactory.NewWithMinMax(SciChart.YAxes[1].VisibleRange, -maxExtent, maxExtent);
        }
      }
    }

    /// <summary>
    /// Setzen der Koordinaten der BoxAnnotation für die Verschiebewerte
    /// </summary>
    /// <param name="data">Aktuelles Element</param>
    /// <param name="corr">Elementliste</param>
    public void SetCoordCH0(TestElement data, TestList corr)
    {
      double maxExtent = 1;   //min max yAxis;1 is default
      if (DataCH0 != null)
      {
        if (DataCH0.Count > 0)
        {
          DoubleRange range = DataCH0.YRange.AsDoubleRange();
          maxExtent = Math.Max(Math.Abs(range.Min), Math.Abs(range.Max));
        }

        Coord_CH0.X1 = data.StartPosition;
        Coord_CH0.X2 = data.StartPosition + 10;

        Coord_CH0.Y1 = maxExtent;
        Coord_CH0.Y2 = -maxExtent;

        if (maxExtent > Double.Epsilon && !Double.IsInfinity(maxExtent) && SciChart.YAxes.Count > 0)
        {
          SciChart.YAxes[0].VisibleRange = RangeFactory.NewWithMinMax(SciChart.YAxes[0].VisibleRange, -maxExtent, maxExtent);
          SciChart.YAxes[0].VisibleRangeLimit = RangeFactory.NewWithMinMax(SciChart.YAxes[0].VisibleRange, -maxExtent, maxExtent);
        }
      }
    }
  }
}